const path = require('path');
const webpack = require('webpack');
const webpackBase = require('./webpack.config.base');
const merge = require('webpack-merge');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin');
const resolve = dir => path.join(__dirname, dir);

module.exports = merge(webpackBase, {
  entry: resolve('../src/examples/index.js'),
  output: {
    path: resolve('../dist'),
    filename: 'build.[hash:8].js',
  },
  devtool: '#inline-source-map',
  module: {
    rules: [{
      test: /\.css$/,
      use: ['style-loader', 'css-loader']
    }, {
      test: /\.less$/,
      use: ['style-loader', 'css-loader', 'less-loader']
    },
    {
      test: /\.(gif|jpg|png|woff|svg|eot|ttf)\??.*$/,
      loader: 'url-loader?limit=50000&name=assets/[name].[ext]'
    }]
  },
  plugins: [
    new HtmlWebpackPlugin({
      filename: 'index.html',
      template: 'src/examples/index.html',
      inject: true
    }),
    new webpack.HotModuleReplacementPlugin(),
    new FriendlyErrorsPlugin()
  ]
})
