import { commonUtils } from '@/index';
import is from 'is_js';
import moment from 'moment';
describe('common-utils.recurseData', () => {
  it('递归处理数据,除原始类型外', () => {
    let data = [{ birth: new Date(1992, 10, 16), son: [{ name: 'nick', birth: new Date(2017, 10, 16) }] }];
    data = commonUtils.recurseData(data, (v, k) => {
      if (is.date(v)) {
        return moment(v).format('YYYY-MM-DD');
      }
      return v;
    });
    expect(data[0].birth).toEqual('1992-11-16');
    expect(data[0].son[0].birth).toEqual('2017-11-16');
  });
});

describe('common-utils.recurseItem', () => {
  let data = [{ name: 'w', age: 25, children: [{ name: 'c', age: 1 }], sons: [{ name: 'c', age: 2 }] }];
  it('递归children属性 ', () => {
    data = commonUtils.recurseItem(data, item => {
      item.age += 1;
      return item;
    });
    expect(data[0].age).toEqual(26);
    expect(data[0].children[0].age).toEqual(2);
  });

  it('递归sons属性 ', () => {
    data = commonUtils.recurseItem(data, item => {
      item.age += 1;
      return item;
    }, 'sons');
    expect(data[0].age).toEqual(27);
    expect(data[0].sons[0].age).toEqual(3);
  });
});

describe('common-utils.uuid', () => {
  it('uuid生成', () => {
    let uuid1 = commonUtils.uuid();
    let uuid2 = commonUtils.uuid();
    expect(uuid1.length).toEqual(36);
    expect(uuid1 === uuid2).toBeFalsy();
  });
});
