import Vue from 'vue';
import { enhanceComp as enhance } from '@/index';

describe('enhance', () => {
  const testWord = "wangchao";
  let beforeInvokeCount = 0;
  let afterInvokeCount = 0;
  let beforeAspectArgs = null;
  let afterAspectArgs = null;
  let afterAspectRet = null;
  let flag = false;

  let vue = new Vue(enhance({
    extends: {
      aspects: {
        beforeChangeName(name) {
          beforeInvokeCount++;
        },
        afterChangeName(res, name) {
          afterInvokeCount++;
        }
      }
    },
    mixins: [{
      aspects: {
        beforeChangeName(name) {
          beforeInvokeCount++;
        },
        afterChangeName(res, name) {
          afterInvokeCount++;
        }
      }
    }],
    data() {
      return {
        name: null
      }
    },
    methods: {
      changeName(name) {
        this.name = name;
        return name;
      }
    },
    aspects: {
      beforeChangeName(name) {
        beforeInvokeCount++;
        if (flag) {
          beforeAspectArgs = name;
        }
      },
      afterChangeName(ret, name) {
        afterInvokeCount++;
        if (flag) {
          afterAspectArgs = name;
          afterAspectRet = ret;
        }
      }
    }
  }));
  it('测试是否混淆、继承和配置中都执行了拦截函数', () => {
    vue.changeName(testWord);
    expect(beforeInvokeCount).toEqual(3);
    expect(afterInvokeCount).toEqual(3);
  })
  it('测试参数是否正确', () => {
    flag = true;
    vue.changeName(testWord);
    expect(beforeAspectArgs).toEqual(testWord);
    expect(afterAspectArgs).toEqual(testWord);
    expect(afterAspectRet).toEqual(testWord);
  })
});

